/*
SRandomStrategy.java              2002/01/31

A class describing an agent's strategy, which buys or sells randomly.
The limit price is set randomly around the latest spot price, and
quantity of the order is set randomly within a prescribed range.
Position of the agent is also considered in decision making.
 */

package strategy;

import java.util.*;

public class MySRandomStrategy extends Strategy {

  /** Default value of Order Price Width */
  public static final int DEFAULT_WIDTH_OF_PRICE = 20;

  /** Default value of Maximum Order Volume */
  public static final int DEFAULT_MAX_QUANT = 50;

  /** Default value of Minimum Order Volume */
  public static final int DEFAULT_MIN_QUANT = 10;

  /** Default value of Maximum Position */
  public static final int DEFAULT_MAX_POSITION = 300;

  /** Default Value of the Price when All the Previous Prices Are Not Defined */
  public static final int DEFAULT_NOMINAL_PRICE = 3000;

  /** Random Number Generator */
  private Random fRandom;

  /** Width of Order Price */
  private int fWidthOfPrice = DEFAULT_WIDTH_OF_PRICE;

  /** Maximum Value of Order Volume */
  private int fMaxQuant = DEFAULT_MAX_QUANT;

  /** Minimum Value of Order Volume */
  private int fMinQuant = DEFAULT_MIN_QUANT;

  /** Maximum Position */
  private int fMaxPosition = DEFAULT_MAX_POSITION;

  /** Price when All the Previous Prices Are Not Defined */
  private int fNominalPrice = DEFAULT_NOMINAL_PRICE;


  public MySRandomStrategy(int seed) {
    fRandom = new Random(seed);
  }

  public Order getOrder(int[] spotPrices, int[] futurePrices,
                        int pos, long money, int restDay) {
    Order order = new Order();
    // Scan a well defined latest futures price. If it is not available,
    // a nominal value is used.
    int prevPrice = getLatestPrice(spotPrices);
    if (prevPrice == -1) {
      prevPrice = fNominalPrice;
    }
    // Submit a random order with a random price around the latest spot price.
    order.buysell = fRandom.nextInt(2) + 1;
    // Cancel decision if it may increase absolute value of the position
    if (order.buysell == Order.BUY) {
      if (pos > fMaxPosition) {
        order.buysell = Order.NONE;
        return order;
      }
    } else if (order.buysell == Order.SELL) {
      if (pos < -fMaxPosition) {
        order.buysell = Order.NONE;
        return order;
      }
    }
    while (true) {
      order.price = prevPrice + (int) (fWidthOfPrice * fRandom.nextGaussian());
      if (order.price > 0)
        break;
    }
    order.quant = fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);

    message("MySRandomStrategy: buysell = " + order.buysell
             + ", price = " + order.price
             + ", volume = " + order.quant
             + " (prevSpotPrice = " + prevPrice + "  )");

    return order;
  }

  /* (non-Javadoc)
   * @see strategy.UBaseStrategy#setParameters(java.lang.String[])
   */
  public void setParameters(String[] args) {
    super.setParameters(args);
    for ( int i = 0; i < args.length; ++i ) {
      StringTokenizer st = new StringTokenizer(args[i], "= ");
      String key = st.nextToken();
      String value = st.nextToken();
      if (  key.equals("WidthOfPrice") ) {
        fWidthOfPrice = Integer.parseInt(value);
      } else if ( key.equals("MinQuant") ) {
        fMinQuant = Integer.parseInt(value);
      } else if ( key.equals("MaxQuant") ) {
        fMaxQuant = Integer.parseInt(value);
      } else if ( key.equals("MaxPosition") ) {
        fMaxPosition = Integer.parseInt(value);
      } else {
        message("Unknown parameter:" + key + " in SRandomStrategy.setParameters");
      }
    }
  }

}
